@@ -19,9 +19,9 @@
 #endif
 
 /*
- * Take a single Kerberos v5 credential and serialize it into a buffer, using
- * the encoding required for putting it into tokens.  output will be a pointer
- * to newly allocated memory, and length will be set to the encoded length.
+ * Take a single Kerberos credential and serialize it into a buffer, using the
+ * encoding required for putting it into tokens.  output will be a pointer to
+ * newly allocated memory, and length will be set to the encoded length.
  * expiration will be set to the expiration time of the ticket.  Returns a
  * WA_ERR code.
  */
@@ -208,7 +208,7 @@ cred_to_attr_encoding(WEBAUTH_KRB5_CTXTP *c, krb5_creds *creds,
  * credential.  Takes the encoded credential string and its length.
  */
 static int
-cred_from_attr_encoding(WEBAUTH_KRB5_CTXTP *c, char *input,
+cred_from_attr_encoding(WEBAUTH_KRB5_CTXTP *c, const void *input,
                         size_t input_length, krb5_creds *creds)
 {
     WEBAUTH_ATTR_LIST *list;
