@@ -9,8 +9,8 @@
  */
 
 #include <modules/mod-config.h>
+#include <portable/apr.h>
 
-#include <apr_strings.h>
 #include <httpd.h>
 #include <http_core.h>
 #include <http_log.h>
