@@ -9,10 +9,9 @@
  */
 
 #include <modules/mod-config.h>
+#include <portable/apr.h>
 
 #include <apr_hash.h>
-#include <apr_pools.h>
-#include <apr_strings.h>
 #include <httpd.h>
 #include <http_config.h>
 #include <http_log.h>
