@@ -2439,10 +2439,14 @@ handler_hook(request_rec *r)
         return DECLINED;
     }
     if (rc.sconf->userinfo_config != NULL) {
-        rc.sconf->userinfo_config->identity = rc.sconf->userinfo_principal;
-        rc.sconf->userinfo_config->keytab = rc.sconf->keytab_path;
-        rc.sconf->userinfo_config->principal = rc.sconf->keytab_principal;
-        status = webauth_user_config(rc.ctx, rc.sconf->userinfo_config);
+        struct webauth_user_config *user = rc.sconf->userinfo_config;
+
+        user->identity       = rc.sconf->userinfo_principal;
+        user->timeout        = rc.sconf->userinfo_timeout;
+        user->ignore_failure = rc.sconf->userinfo_ignore_fail;
+        user->keytab         = rc.sconf->keytab_path;
+        user->principal      = rc.sconf->keytab_principal;
+        status = webauth_user_config(rc.ctx, user);
         if (status != WA_ERR_NONE) {
             ap_log_error(APLOG_MARK, APLOG_CRIT, 0, r->server,
                          "mod_webkdc: webauth_user_config failed: %s",
@@ -2514,7 +2518,6 @@ mod_webkdc_init(apr_pool_t *pconf, apr_pool_t *plog UNUSED,
 {
     struct config *sconf;
     server_rec *scheck;
-    char *version;
 
     sconf = ap_get_module_config(s->module_config, &webkdc_module);
 
@@ -2530,14 +2533,12 @@ mod_webkdc_init(apr_pool_t *pconf, apr_pool_t *plog UNUSED,
         webkdc_config_init(scheck, sconf, ptemp);
     }
 
-    version = apr_pstrcat(ptemp, "WebKDC/", webauth_info_version(), NULL);
-    ap_add_version_component(pconf, version);
+    ap_add_version_component(pconf, "WebKDC/" PACKAGE_VERSION);
 
     if (sconf->debug)
         ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, s,
-                     "mod_webkdc: initialized (%s) (%s)",
-                     webauth_info_version(),
-                     webauth_info_build());
+                     "mod_webkdc: initialized (%s) (%s)", VERSION,
+                     PACKAGE_BUILD_INFO);
 
     return OK;
 }
