@@ -9,10 +9,9 @@
  */
 
 #include <modules/mod-config.h>
+#include <portable/apr.h>
 
 #include <apr_errno.h>
-#include <apr_pools.h>
-#include <apr_strings.h>
 #include <apr_thread_mutex.h>
 #include <ap_config.h>
 #include <httpd.h>
